/*///////////////////////////

      Soda Fountain

///////////////////////////*/



///////////Libraries///////////

#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

#include <Keypad.h>

#include <DFRobot_HX711.h>


////////Global Variables////////

//Screen
LiquidCrystal_I2C lcd(0x27,16,2);  // set the LCD address to 0x27 for a 16 chars and 2 line display


//Keyboard
int cursorColumn = 0;
const byte ROWS = 4; 
const byte COLS = 3; 

char hexaKeys[ROWS][COLS] = {
  {'1', '2', '3'},
  {'4', '5', '6'},
  {'7', '8', '9'},
  {'*', '0', '#'}
};

byte rowPins[ROWS] = {7, 12, 11,9 }; 
byte colPins[COLS] = {8, 6, 10}; 

Keypad customKeypad = Keypad(makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS); 


//Weight sensor 
DFRobot_HX711 MyScale(A1, A2);

//State Machine
int currentState=0;

/*///////////////////////////
Variables Declaration
///////////////////////////*/

#define drink1_mW (totalMoneyWeight+(coinWeight*drink1Price*halfFull_Price))
#define drink2_mW (totalMoneyWeight+(coinWeight*drink2Price*halfFull_Price))
#define drink3_mW (totalMoneyWeight+(coinWeight*drink3Price*halfFull_Price))

#define glass1_Present 680
#define glass2_Present 780
#define glass3_Present 750

#define halfCupCounter 3
#define fullCupCounter 5

int full_Cup=0;

int pumpOn=0;
int pumpOn_counter=0;

int prev_pumpState=1;


int drink = 0;
const int disp1_sensor=A3;
int disp1_sensor_value=0;

const int disp2_sensor=A4;
int disp2_sensor_value=0;

const int disp3_sensor=A5;
int disp3_sensor_value=0;

const int money_sensor=A1;
int money_sensor_value=0;

int coinWeight=6;

int drink1Price=2;
int drink2Price=4;
int drink3Price=4;

float halfFull_Price=1;
int totalMoneyWeight=0;
int prev_MoneyState=0;

const int disp1_pump=2;
const int disp2_pump=3;
const int disp3_pump=4;

int welcome_rutine=1;


void setup()
{
  
  // set up the LCD's number of columns and rows:
  lcd.init();                      // initialize the lcd 
  lcd.init();
  // Print a message to the LCD.
  lcd.backlight();
  
  // Print a message to the LCD.
  lcd.setCursor(1, 0); 
  lcd.print("Soda Fountain");
  
  //  setup serial
  Serial.begin(9600);           
  pinMode(disp1_sensor, INPUT);
  pinMode(disp1_pump,OUTPUT);

  pinMode(disp2_sensor, INPUT);
  pinMode(disp2_pump,OUTPUT);

  pinMode(disp3_sensor, INPUT);
  pinMode(disp3_pump,OUTPUT);

  digitalWrite(disp1_pump,LOW);
  digitalWrite(disp2_pump,LOW);
  digitalWrite(disp3_pump,LOW);
  
}

void loop()
{  
  char key;

  //Read the keyboard 
  key = customKeypad.getKey();

Serial.println(key);
  //Return to Welcome Screen if the user cancels the operation
  if (key)
    {
      if (key=='#')
      {
        currentState=0;
      }
    }
  
  switch (currentState)
  {

    /*Welcome Screen*/
    case 0:

    // Display Welcome Screen
    lcd.setCursor(0, 0); 
    lcd.clear();
    display_message(1,0,"Soda Fountain!"); 
    delay(500);
    
    display_message(4,1,"by Dr. C"); 
    delay(300);

    // Change the current State to Start Soda Fountain operation 
    currentState = 400;


    /*Start Soda Fountain operation*/
    case 400:

    //Read the keyboard
    //key = customKeypad.getKey();
    //Serial.println(key); 
  
  
    // Wait until the user starts the operation by pressing *
    if (key)
    {

      if (key=='*')
      {
        
        //Read the current weight of the money plate and save it as reference
        totalMoneyWeight = MyScale.readWeight();  

        //Modify the required money constant for each of the drinks according to the current totalMoneyWeight 
        //Serial.println("_________");
        //Serial.println(totalMoneyWeight); 
        //Serial.println(drink1_mW); 

        //Change Current State to Drink Selection
        currentState=1;
        prev_MoneyState=0;
        welcome_rutine=1;
        lcd.clear();
      }
    }
    break;

    /*Drink Selection*/
    case 1:

    //Display the Drink Selection message
    display_message(3,0,"Select your"); 
    display_message(6,1,"drink"); 

    //Wait for the user to select a valid drink, if the selection if valid go to Half or Full Cup Selection Selection
    if (key)
    {
      currentState=2;
      lcd.clear();
      
      if (key=='1')
      {
        drink=1;

        //Save and display on the screen user's selection
        display_message(3,0,"Marianita's"); 
        display_message(5,1,"Water"); 
        delay(2000);
      }
      else if (key=='2')
      {
        drink=2;

        //Save and display on the screen user's selection
        display_message(3,0,"Brittany's"); 
        display_message(6,1,"Cola"); 
        delay(2000);
      }
      else if (key=='3')
      {
        //Save and display on the screen user's selection
        drink=3;

        display_message(4,0,"Michel's"); 
        display_message(4,1,"Ice Tea"); 
        delay(2000);
      }
      else
      {
        //Invalid selection, stay in current State
        currentState=1;
      }
      
      lcd.clear();
    }
    break;


   /*Half or Full Cup Selection*/ 
   case 2:
   
    //Display the Half or Full Cup Selection Message
    display_message(4,0,"Half/Full"); 
    display_message(7,1,"Cup"); 

    //Wait for the user to select either Half or Full cup, if the selection if valid go to Payment display
    if (key)
    {
      currentState=3;
      lcd.clear();
      if (key=='0')
      {
        //Save the Half cup expected weight value and display user's selection
        full_Cup = halfCupCounter;
        halfFull_Price = 0.5;
        
        display_message(6,0,"Half"); 
        display_message(6,1,"Cup");
        delay(2000);
      }
      else if (key=='1')
      {
        //Save the Full cup expected weight value and display user's selection
        full_Cup=fullCupCounter; 
        halfFull_Price=1;
               
        display_message(6,0,"Full"); 
        display_message(6,1,"Cup");
        delay(2000);
      }
      else
      {
        //Invalid selection, stay in current State
        currentState=2;
      }
      lcd.clear();    
    }
    break;

    /*Payment display*/
    case 3:

    //Display the Payment Display Message
    display_message(3,0,"Introduce:");

    //Weight the money tray
    money_sensor_value = MyScale.readWeight();  // read the input pin  
    Serial.println(money_sensor_value);
    Serial.println("_");

    if (money_sensor_value>0 || prev_MoneyState==0)
    {
      //Compare the money_sensor_value to the expected weight and print the amount of money to introduce
      if (drink==1 && money_sensor_value < drink1_mW)
      {
        display_message(6,1,String(drink1Price*halfFull_Price)); 
        display_message(7,1,".00"); 
        prev_MoneyState=1;
      }
      else if (drink==2 && money_sensor_value < drink2_mW)
      {
        display_message(6,1,String(drink2Price*halfFull_Price)); 
        display_message(7,1,".00"); 
        prev_MoneyState=1;
      }
      else if (drink==3 && money_sensor_value < drink3_mW)
      {
        display_message(6,1,String(drink3Price*halfFull_Price)); 
        display_message(7,1,".00");
        prev_MoneyState=1;
      }
      else
      {
        //Once the drink is paid change to Filling drink State
        currentState=4;
        lcd.clear();  
        prev_MoneyState=0;
      }
    }
    break;
    
    /*Filling drink State*/
    case 4:

    //Read the corresponding dispenser sensor to know the amount of drink to pour
    disp1_sensor_value = analogRead(disp1_sensor); 
    disp2_sensor_value = analogRead(disp2_sensor); 
    disp3_sensor_value = analogRead(disp3_sensor); 
    //Serial.println(disp2_sensor_value); 
    
    //If there is no glass ask the user to put the glass in the right dispenser
    switch(drink)
    {
      case 1:
      //If there is no glass stop the pump and display the Put your drink message
      if (disp1_sensor_value > glass1_Present)
      {
        //Display the Put your drink message if the pump was previously on or the user just enter Filling drink State 
        if(prev_pumpState==1||welcome_rutine==1)
        {
          lcd.clear();  
          pump_control(disp1_sensor, disp1_pump, glass1_Present);
          display_message(4,0,"Put your"); 
          display_message(6,1,"glass"); 
          prev_pumpState=0;
          welcome_rutine=0;
        }
      }
      //Start filling the glass with water
      else
      {
        display_message(2,0,"Filling with"); 
        display_message(6,1,"Water"); 
        pump_control(disp1_sensor, disp1_pump, glass1_Present);
        
      }
      break;
      
      case 2:
      //If there is no glass stop the pump and display the Put your drink message
      if (disp2_sensor_value > glass2_Present)
      {
        //Display the Put your drink message if the pump was previously on or the user just enter Filling drink State 
        if(prev_pumpState==1||welcome_rutine==1)
        {
          lcd.clear();  
          pump_control(disp2_sensor, disp2_pump, glass2_Present);
          display_message(4,0,"Put your"); 
          display_message(6,1,"glass"); 
          prev_pumpState=0;
          welcome_rutine=0;
        }
      }
      //Start filling the glass with Cola
      else
      {
        display_message(2,0,"Filling with"); 
        display_message(6,1,"Cola"); 
        pump_control(disp2_sensor, disp2_pump, glass2_Present); 
      }
      break;
      
      case 3:
      //If there is no glass stop the pump and display the Put your drink message
      if (disp3_sensor_value > glass3_Present)
      {
        //Display the Put your drink message if the pump was previously on or the user just enter Filling drink State 
        if(prev_pumpState==1||welcome_rutine==1)
        {
          lcd.clear();  
          pump_control(disp3_sensor, disp3_pump, glass3_Present);
          display_message(4,0,"Put your"); 
          display_message(6,1,"glass"); 
          prev_pumpState=0;
          welcome_rutine=0;
        }
      }
      //Start filling the glass with Ice tea
      else
      {
        display_message(2,0,"Filling with"); 
        display_message(5,1,"Ice Tea"); 
        pump_control(disp3_sensor, disp3_pump, glass3_Present);
      }
      break;

      default:
      lcd.clear();
      break;
    }
    break;

    //Goodbye Screen
    case 5:

    //Display Goodbye Screen
    display_message(3,0,"Enjoy your"); 
    display_message(6,1,"drink");
    prev_pumpState=1; 
    delay(1500);

    //Go back to Welcome Screen
    currentState=0;
    break;
    
    default:
    currentState=0;
    break;
  }
  
}
void display_message(int col, int ln, String message)
{
  // set the cursor
  lcd.setCursor(col, ln);
  // print message
  lcd.print(message);
}
  
void pump_control(int n_dispenser, int n_pump, int glass_Present)
{
  //Read the selected sensor value
  int disp_sensor_value = analogRead(n_dispenser);

  //Turn off the pump if there is no glass else turn on the pump for full_Cup time
  if(disp_sensor_value > glass_Present)
  {
    digitalWrite(n_pump,LOW);
    lcd.clear();  
  }
  else
  {
    //While the timer is less than full cup turn on the pump
    if (pumpOn_counter<=full_Cup)
    {
      Serial.println(pumpOn_counter); 
      digitalWrite(n_pump,HIGH);
      delay(3000);
      pumpOn_counter++;
      prev_pumpState=1;
    }
    //Once the pumpOn_counter reaches the full_Cup time turn off the pump
    else
    {
      //Serial.println(disp1_sensor_value); 
      digitalWrite(n_pump,LOW);
      pumpOn_counter=0;
      
      lcd.clear();
      currentState=5;
      prev_pumpState=0;
    }
    
  }
  
}
